#include <GL/glew.h>
#include <GL/freeglut.h>

#include <iostream>

#include "framebuffer.hpp"
#include "shader.hpp"
#include "texture.hpp"
#include "quad.hpp"

#define MIN(X,Y) ((X) < (Y) ? (X) : (Y))
#define MAX(X,Y) ((X) > (Y) ? (X) : (Y))

int windowWidth = 600;
int windowHeight = 600;
int windowScale = 8;

Quad* fullscreenQuad;
Texture2D* noiseTexture;
Shader* simpleShader;

void init(){

	glewExperimental = GL_TRUE;
	GLenum err = glewInit();
	if (GLEW_OK != err)	{
		std::cerr << "Error: "<< glewGetErrorString(err) << std::endl;
	} else {
		if (GLEW_VERSION_3_3)
		{
			std::cout<< "Driver supports OpenGL 3.3\nDetails:"<< std::endl;
			std::cout << "Using GLEW " << glewGetString(GLEW_VERSION)<< std::endl;
			std::cout << "Vendor: " << glGetString (GL_VENDOR)<< std::endl;
			std::cout << "Renderer: " << glGetString (GL_RENDERER)<< std::endl;
			std::cout << "Version: " << glGetString (GL_VERSION)<< std::endl;
			std::cout << "GLSL: " << glGetString (GL_SHADING_LANGUAGE_VERSION)<< std::endl;
		}
	}
	glGetError();


	fullscreenQuad = new Quad();
	simpleShader = new Shader("passthrough.vert", "simple.frag");

	noiseTexture = new Texture2D();
	noiseTexture->loadFromFile(std::string("noise.tga"));

}

void display(){

	glClearColor(0.17f, 0.4f, 0.6f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	simpleShader->enable();
	simpleShader->bindUniformTexture("inputTex", noiseTexture->getTextureHandle(),0);
	fullscreenQuad->render(simpleShader);
	simpleShader->disable();

	glutSwapBuffers();
}

void keyboard(unsigned char key, int x, int y){
	switch(key){

	case 27:
		exit(0);
		break;

	default:
		std::cout << "Unbinded key: " << (unsigned int)key << std::endl;
	}

	glutPostRedisplay();
}

void reshape(int width, int height){
	windowWidth = width;
	windowHeight = height;
	glViewport(0, 0, width, height);
}

void mouse(int button, int state, int x, int y)
{
	if(GLUT_LEFT_BUTTON == button && GLUT_UP == state)
	{
	}
	else if(GLUT_RIGHT_BUTTON == button && GLUT_UP == state)
	{
	}
}

int main(int argc, char* argv[]){
	glutInit(&argc, argv);
	glutInitContextVersion (3, 0);
	glutInitContextFlags (GLUT_CORE_PROFILE | GLUT_DEBUG);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowSize(windowWidth, windowHeight);
	glutCreateWindow("GPGPU 3. labor: glsl");

	init();

	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(keyboard);
	glutMouseFunc(mouse);

	glutMainLoop();

	return(0);
}
